@@ -38,7 +38,7 @@ free_author_map (void)
     }
 }
 
-int
+bool
 load_author_map (char *filename)
 /* load author-map information from a file */
 {
@@ -54,8 +54,8 @@ load_author_map (char *filename)
     
     f = fopen (filename, "r");
     if (!f) {
-	fprintf (stderr, "%s: %s\n", filename, strerror (errno));
-	return 1;
+	announce("%s: authormap open failed, %s\n", filename, strerror (errno));
+	return false;
     }
     while (fgets (line, sizeof (line) - 1, f)) {
 	lineno++;
@@ -63,9 +63,9 @@ load_author_map (char *filename)
 	    continue;
 	equal = strchr (line, '=');
 	if (!equal) {
-	    fprintf (stderr, "%s: (%d) missing '='\n", filename, lineno);
+	    announce("\"%s\", line %d: missing '='\n", filename, lineno);
 	    fclose (f);
-	    return 0;
+	    return false;
 	}
 	full = equal + 1;
 	while (equal > line && equal[-1] == ' ')
@@ -73,20 +73,20 @@ load_author_map (char *filename)
 	*equal = '\0';
 	name = atom (line);
 	if (fullname (name)) {
-	    fprintf (stderr, "%s: (%d) duplicate name '%s' ignored\n",
+	    announce("\"%s\", line %d: duplicate username '%s' ignored\n",
 		     filename, lineno, name);
 	    fclose (f);
 	    return 0;
 	}
-	a = calloc (1, sizeof (cvs_author));
+	a = xcalloc (1, sizeof (cvs_author), "authormap creation");
 	a->name = name;
 	angle = strchr (full, '<');
 	if (!angle) {
-	    fprintf (stderr, "%s: (%d) missing email address '%s'\n",
+	    announce("\"%s\", line %d: missing email address '%s'\n",
 		     filename, lineno, name);
 	    fclose (f);
 	    free(a);
-	    return 0;
+	    return false;
 	}
 	email = angle + 1;
 	while (full < angle && full[0] == ' ')
@@ -97,21 +97,21 @@ load_author_map (char *filename)
 	a->full = atom(full);
 	angle = strchr (email, '>');
 	if (!angle) {
-	    fprintf (stderr, "%s: (%d) malformed email address '%s\n",
+	    announce("\"%s\", line %d: malformed email address '%s\n",
 		     filename, lineno, name);
 	    fclose (f);
 	    free(a);
-	    return 0;
+	    return false;
 	}
 	*angle = '\0';
 	a->email = atom (email);
 	a->timezone = NULL;
 	if (*++angle) {
-	    while (isspace(*angle))
+	    while (isspace((unsigned char)*angle))
 		angle++;
 	    while (*angle != '\0') {
 		char *end = angle + strlen(angle) - 1;
-		if (isspace(*end))
+		if (isspace((unsigned char)*end))
 		    *end = '\0';
 		else
 		    break;
@@ -123,7 +123,7 @@ load_author_map (char *filename)
 	*bucket = a;
     }
     fclose (f);
-    return 1;
+    return true;
 }
 
 /* end */
