@@ -21,13 +21,17 @@
 #include <sys/types.h>
 #include <errno.h>
 #include <getopt.h>
+#include <regex.h>
+#if defined(__GLIBC__)
+#include <malloc.h>
+#endif /* __GLIBC__ */
 
 #ifndef MAXPATHLEN
 #define MAXPATHLEN  10240
 #endif
 
 typedef enum _rev_execution_mode {
-    ExecuteExport, ExecuteGraph, ExecuteSplits
+    ExecuteExport, ExecuteGraph,
 } rev_execution_mode;
 
 /* options */
@@ -36,6 +40,11 @@ bool force_dates = false;
 bool suppress_keyword_expansion = false;
 bool reposurgeon;
 FILE *revision_map;
+char *branch_prefix = "refs/heads/";
+bool progress = false;
+bool branchorder = false;
+time_t start_time;
+
 static int verbose = 0;
 static rev_execution_mode rev_mode = ExecuteExport;
 
@@ -43,15 +52,12 @@ char *
 stringify_revision (char *name, char *sep, cvs_number *number)
 /* stringify a revision number */
 {
-    static char result[BUFSIZ], digits[32];
+    static char result[BUFSIZ];
 
     if (name != NULL)
     {
 	if (strlen(name) >= sizeof(result) - strlen(sep) - 1)
-	{
-	    fprintf(stderr, "Filename too long\n");
-	    exit(1);
-	}
+	    fatal_error("filename too long");
 	strncpy(result, name, sizeof(result) - strlen(sep) - 1);
 	strcat(result, sep);
     }
@@ -59,13 +65,12 @@ stringify_revision (char *name, char *sep, cvs_number *number)
     if (number) 
     {
 	int i;
+	char digits[32];
+
 	for (i = 0; i < number->c; i++) {
 	    snprintf (digits, sizeof(digits)-1, "%d", number->n[i]);
 	    if (strlen(result) + 1 + strlen(digits) >= sizeof(result))
-	    {
-		fprintf(stderr, "Revision number too long\n");
-		exit(1);
-	    }
+		fatal_error("Revision number too long\n");
 	    strcat(result, digits);
 	    if (i < number->c - 1)
 		strcat (result, ".");
@@ -89,8 +94,10 @@ dump_number (char *name, cvs_number *number)
     dump_number_file (stdout, name, number);
 }
 
+#ifdef __UNUSED__
 void
-dump_rev_commit (rev_commit *c)
+dump_git_commit (git_commit *c)
+/* dump all delta/revision pairs associated with a gitspace commit */
 {
     rev_file	*f;
     int		i, j;
@@ -109,10 +116,11 @@ dump_rev_commit (rev_commit *c)
 
 void
 dump_rev_head (rev_ref *h)
+/* dump all gitspace commits associated wit the specified head */
 {
-    rev_commit	*c;
+    git_commit	*c;
     for (c = h->commit; c; c = c->parent) {
-	dump_rev_commit (c);
+	dump_git_commit (c);
 	if (c->tail)
 	    break;
     }
@@ -120,6 +128,7 @@ dump_rev_head (rev_ref *h)
 
 void
 dump_rev_list (rev_list *rl)
+/* dump an entire revision list */
 {
     rev_ref	*h;
 
@@ -129,18 +138,14 @@ dump_rev_list (rev_list *rl)
 	dump_rev_head (h);
     }
 }
-
-typedef struct _rev_split {
-    struct _rev_split	*next;
-    rev_commit		*childa, *childb;
-    rev_commit		*parent;
-    rev_commit		*topa, *topb;
-} rev_split;
+#endif /* __UNUSED__ */
 
 char *
-ctime_nonl (time_t *date)
+ctime_nonl (cvstime_t *date)
+/* ctime(3) with trailing \n removed */
 {
-    char	*d = ctime (date);
+    time_t	udate = RCS_EPOCH + *date;
+    char	*d = ctime (&udate);
     
     d[strlen(d)-1] = '\0';
     return d;
@@ -163,10 +168,11 @@ rev_list_file (char *name, int *nversions)
     if (!yyin) {
 	perror (name);
 	++err;
+	return NULL;
     }
     yyfilename = name;
     yylineno = 0;
-    this_file = calloc (1, sizeof (cvs_file));
+    this_file = xcalloc (1, sizeof (cvs_file), __func__);
     this_file->name = name;
     if (yyin)
 	assert (fstat (fileno (yyin), &buf) == 0);
@@ -183,110 +189,13 @@ rev_list_file (char *name, int *nversions)
     return rl;
 }
 
-void
-dump_splits (rev_list *rl)
-{
-#if 0
-    rev_split	*splits = NULL, *s;
-    rev_ref	*head;
-    rev_commit	*c, *a, *b;
-    int		ai, bi;
-    rev_file	*af, *bf;
-    char	*which;
-
-    /* Find tails and mark splits */
-    for (head = rl->heads; head; head = head->next) {
-	if (head->tail)
-	    continue;
-	for (c = head->commit; c; c = c->parent)
-	    if (c->tail) {
-		for (s = splits; s; s = s->next)
-		    if (s->parent == c->parent)
-			break;
-		if (!s) {
-		    s = calloc (1, sizeof (rev_split));
-		    s->parent = c->parent;
-		    s->childa = c;
-		    s->topa = head->commit;
-		    s->next = splits;
-		    splits = s;
-		}
-	    }
-    }
-    /* Find join points */
-    for (s = splits; s; s = s->next) {
-	for (head = rl->heads; head; head = head->next) {
-	    if (head->tail)
-		continue;
-	    for (c = head->commit; c; c = c->parent) {
-		if (c->parent == s->parent && c != s->childa) {
-		    s->childb = c;
-		    s->topb = head->commit;
-		}
-	    }
-	}
-    }
-    for (s = splits; s; s = s->next) {
-	if (s->parent && s->childa && s->childb) {
-	    for (head = rl->heads; head; head = head->next) {
-		if (head->commit == s->topa)
-		    fprintf (stderr, "%s ", head->name);
-	    }
-	    fprintf (stderr, "->");
-	    for (head = rl->heads; head; head = head->next) {
-		if (head->commit == s->topb)
-		    fprintf (stderr, "%s ", head->name);
-	    }
-	    fprintf (stderr, "\n");
-	    a = s->childa;
-	    b = s->childb;
-	    ai = bi = 0;
-	    while (ai < a->nfiles && bi < b->nfiles) {
-		af = a->files[ai];
-		bf = b->files[bi];
-		if (af != bf) {
-		    if (rev_file_later (af, bf)) {
-			fprintf (stderr, "a : %s ", ctime_nonl (&af->date));
-			dump_number_file (stderr, af->name, &af->number);
-			ai++;
-		    } else {
-			fprintf (stderr, " b: %s ", ctime_nonl (&bf->date));
-			dump_number_file (stderr, bf->name, &bf->number);
-			bi++;
-		    }
-		    fprintf (stderr, "\n");
-		} else {
-//		    fprintf (stderr, "ab: %s ", ctime_nonl (&af->date));
-//		    dump_number_file (stderr, af->name, &af->number);
-//		    fprintf (stderr, "\n");
-		    ai++;
-		    bi++;
-		}
-	    }
-	    which = "a ";
-	    if (ai >= a->nfiles) {
-		a = b;
-		ai = bi;
-		which = " b";
-	    }
-	    while (ai < a->nfiles) {
-		af = a->files[ai];
-		fprintf (stderr, "%s: ", which);
-		dump_number_file (stderr, af->name, &af->number);
-		fprintf (stderr, "\n");
-		ai++;
-	    }
-	}
-    }
-#endif
-}
-
+#ifdef __UNUSED__
 void
 dump_rev_tree (rev_list *rl)
 {
     rev_ref	*h;
     rev_ref	*oh;
-    rev_commit	*c, *p;
+    git_commit	*c, *p;
     int		tail;
 
     printf ("rev_list {\n");
@@ -364,8 +273,6 @@ dump_rev_tree (rev_list *rl)
     printf ("}\n");
 }
 
-time_t	time_now;
-
 static int
 strcommon (char *a, char *b)
 /* return the length of the common prefix of strings a and b */
@@ -381,6 +288,141 @@ strcommon (char *a, char *b)
     }
     return c;
 }
+#endif /* __UNUSED__ */
+
+static int
+strcommonendingwith(char *a, char *b, char endc)
+/* return the length of the common prefix of strings a and b ending with endc */
+{
+    int c = 0;
+    int d = 0;
+    while (*a == *b) {
+	if (!*a) {
+	    break;
+ 	}
+	a++;
+	b++;
+	c++;
+	if (*a == endc) {
+	    d = c + 1;
+	}
+    }
+    return d;
+}
+
+static int get_int_substr(const char * str, const regmatch_t * p)
+{
+    char buff[256];
+    if(p->rm_so == -1)
+	    return 0;
+    if(p->rm_eo - p->rm_so >= sizeof(buff))
+	    return 0;
+    memcpy(buff, str + p->rm_so, p->rm_eo - p->rm_so);
+    buff[p->rm_eo - p->rm_so] = 0;
+    return atoi(buff);
+}
+
+static time_t mktime_utc(struct tm * tm, const char* tzbuf)
+{
+    /* coverity[tainted_string_return_content] */
+    char * old_tz = getenv("TZ");
+    time_t ret;
+
+    setenv("TZ", tzbuf, 1);
+
+    tzset();
+
+    ret = mktime(tm);
+
+    if (old_tz)
+	setenv("TZ", old_tz, 1);
+    else
+	unsetenv("TZ");
+
+    tzset();
+
+    return ret;
+}
+
+static time_t convert_date(const char *dte)
+/* accept a date in anything close to RFC3339 form */
+{
+    static regex_t date_re;
+    static bool init_re;
+
+#define MAX_MATCH 16
+    size_t nmatch = MAX_MATCH;
+    regmatch_t match[MAX_MATCH];
+
+    if (!init_re)
+    {
+	if (regcomp(&date_re, "([0-9]{4})[-/]([0-9]{2})[-/]([0-9]{2})[ T]([0-9]{2}):([0-9]{2}):([0-9]{2})( [-+][0-9]{4})?", REG_EXTENDED)) 
+	    fatal_error("date regex compilation error\n");
+	init_re = true;
+    }
+
+    if (regexec(&date_re, dte, nmatch, match, 0) == 0)
+    {
+	regmatch_t * pm = match;
+	struct tm tm = {0};
+	char tzbuf[32];
+	int offseth, offsetm;
+
+	/* first regmatch_t is match location of entire re */
+	pm++;
+
+	tm.tm_year = get_int_substr(dte, pm++);
+	tm.tm_mon  = get_int_substr(dte, pm++);
+	tm.tm_mday = get_int_substr(dte, pm++);
+	tm.tm_hour = get_int_substr(dte, pm++);
+	tm.tm_min  = get_int_substr(dte, pm++);
+	tm.tm_sec  = get_int_substr(dte, pm++);
+	offseth    = -get_int_substr(dte, pm++);
+
+	offsetm = offseth % 100;
+	if(offsetm < 0)
+	    offsetm *= -1;
+	offseth /= 100;
+	snprintf(tzbuf, sizeof(tzbuf), "UTC%+d:%d", offseth, offsetm);
+
+	tm.tm_year -= 1900;
+	tm.tm_mon--;
+
+	return mktime_utc(&tm, tzbuf);
+    }
+    else
+    {
+	return atoi(dte);
+    }
+}
+
+static void print_sizes(void)
+{
+    printf("sizeof(char *)        = %zu\n", sizeof(char *));
+    printf("sizeof(long)          = %zu\n", sizeof(long));
+    printf("sizeof(int)           = %zu\n", sizeof(int));
+    printf("sizeof(short)         = %zu\n", sizeof(short));
+    printf("sizeof(cvstime_t)     = %zu\n", sizeof(cvstime_t));
+    printf("sizeof(time_t)        = %zu\n", sizeof(time_t));
+    printf("sizeof(cvs_number)    = %zu\n", sizeof(cvs_number));
+    printf("sizeof(Node)          = %zu\n", sizeof(Node));
+    printf("sizeof(cvs_symbol)    = %zu\n", sizeof(cvs_symbol));
+    printf("sizeof(cvs_branch)    = %zu\n", sizeof(cvs_branch));
+    printf("sizeof(cvs_version)   = %zu\n", sizeof(cvs_version));
+    printf("sizeof(cvs_patch)     = %zu\n", sizeof(cvs_patch));
+    printf("sizeof(cvs_file)      = %zu\n", sizeof(cvs_file));
+    printf("sizeof(rev_file)      = %zu\n", sizeof(rev_file));
+    printf("sizeof(rev_dir)       = %zu\n", sizeof(rev_dir));
+    printf("sizeof(cvs_commit)    = %zu\n", sizeof(cvs_commit));
+    printf("sizeof(git_commit)    = %zu\n", sizeof(git_commit));
+    printf("sizeof(rev_ref)       = %zu\n", sizeof(rev_ref));
+    printf("sizeof(rev_list)      = %zu\n", sizeof(rev_list));
+    printf("sizeof(rev_file_list) = %zu\n", sizeof(rev_file_list));
+    printf("sizeof(rev_diff)      = %zu\n", sizeof(rev_diff));
+    printf("sizeof(cvs_author)    = %zu\n", sizeof(cvs_author));
+    printf("sizeof(Chunk)         = %zu\n", sizeof(Chunk));
+    printf("sizeof(Tag)           = %zu\n", sizeof(Tag));
+}
 
 typedef struct _rev_filename {
     struct _rev_filename	*next;
@@ -401,6 +443,22 @@ main (int argc, char **argv)
     int		    c;
     char	    *file;
     int		    nfile = 0;
+    time_t          fromtime = 0;
+    off_t	    textsize = 0;
+
+#if defined(__GLIBC__)
+    /* 
+     * The default sbrk call grabs memory from the OS in 128kb chunks
+     * for malloc. As we use up memory in prodigous amounts it is
+     * better to grab it in bigger chunks, and make less calls to
+     * sbrk. 16MB seems to be a decent value.
+     */
+    mallopt(M_TOP_PAD,16*1024*1024); /* grab memory in 16MB chunks */
+#endif /* __GLIBC__ */
+    start_time = time(NULL);
+
+    /* force times using mktime to be interpreted in UTC */
+    setenv ("TZ", "UTC", 1);
 
     while (1) {
 	static struct option options[] = {
@@ -410,10 +468,16 @@ main (int argc, char **argv)
 	    { "commit-time-window", 1, 0, 'w' },
 	    { "author-map",         1, 0, 'A' },
 	    { "revision-map",       1, 0, 'R' },
-	    { "reposurgeon",        1, 0, 'r' },
+	    { "reposurgeon",        0, 0, 'r' },
             { "graph",              0, 0, 'g' },
+            { "remote",             1, 0, 'e' },
+            { "strip",              1, 0, 's' },
+            { "progress",           0, 0, 'p' },
+            { "incremental",        1, 0, 'i' },
+            { "branchorder",        0, 0, 'B' },	/* undocumented */
+	    { "sizes",              0, 0, 'S' },	/* undocumented */
 	};
-	int c = getopt_long(argc, argv, "+hVw:grvA:R:T", options, NULL);
+	int c = getopt_long(argc, argv, "+hVw:grvA:R:Tke:s:pi:BS", options, NULL);
 	if (c < 0)
 	    break;
 	switch (c) {
@@ -430,6 +494,10 @@ main (int argc, char **argv)
 		   " -R --revision-map               Revision map file\n"
 		   " -r --reposurgeon                Issue cvs-revision properties\n"
 		   " -T                              Force deteministic dates\n"
+                   " -e --remote                     Relocate branches to refs/remotes/REMOTE\n"
+                   " -s --strip                      Strip the given prefix instead of longest common prefix\n"
+		   " -p --progress                   Enable load-status reporting\n"
+		   " -i --incremental TIME           Incremental dump beginning after specified RFC3339-format time.\n"
 		   "\n"
 		   "Example: find -name '*,v' | cvs-fast-export\n");
 	    return 0;
@@ -464,8 +532,27 @@ main (int argc, char **argv)
 	case 'T':
 	    force_dates = true;
 	    break;
+	case 'e':
+	    branch_prefix = (char*) xmalloc(strlen(optarg) + 15, __func__);
+	    sprintf(branch_prefix, "refs/remotes/%s/", optarg);
+	    break;
+	case 's':
+	    strip = strlen(optarg) + 1;
+	    break;
+	case 'p':
+	    progress = true;
+	    break;
+	case 'i':
+	    fromtime = convert_date(optarg);
+	    break;
+	case 'B':
+	    branchorder = true;
+	    break;
+	case 'S':
+	    print_sizes();
+	    return 0;
 	default: /* error message already emitted */
-	    fprintf(stderr, "Try `%s --help' for more information.\n", argv[0]);
+	    announce("try `%s --help' for more information.\n", argv[0]);
 	    return 1;
 	}
     }
@@ -474,15 +561,14 @@ main (int argc, char **argv)
     argv += optind-1;
     argc -= optind-1;
 
-    /* force times using mktime to be interpreted in UTC */
-    setenv ("TZ", "UTC", 1);
-    time_now = time (NULL);
+    progress_begin("Reading list of files...", NO_MAX);
     for (;;)
     {
 	struct stat stb;
 
 	if (argc < 2) {
 	    int l;
+	    /* coverity[tainted_data] Safe, never handed to exec */
 	    if (fgets (name, sizeof (name) - 1, stdin) == NULL)
 		break;
 	    l = strlen (name);
@@ -499,17 +585,19 @@ main (int argc, char **argv)
 	    continue;
 	else if (S_ISDIR(stb.st_mode) != 0)
 	    continue;
+	else
+	    textsize += stb.st_size;
 
-	fn = calloc (1, sizeof (rev_filename));
+	fn = xcalloc (1, sizeof (rev_filename), "filename gathering");
 	fn->file = atom (file);
 	*fn_tail = fn;
 	fn_tail = &fn->next;
-	if (strip > 0) {
-	    c = strcommon (fn->file, last);
+	if (strip > 0 && last != NULL) {
+	    c = strcommonendingwith (fn->file, last, '/');
 	    if (c < strip)
 		strip = c;
 	} else if (strip < 0) {
-	    int i;
+	    size_t i;
 
 	    strip = 0;
 	    for (i = 0; i < strlen (fn->file); i++)
@@ -518,11 +606,15 @@ main (int argc, char **argv)
 	}
 	last = fn->file;
 	nfile++;
+	if (progress && nfile % 100 == 0)
+	    progress_jump(nfile);
     }
+    progress_end("done, %ldKB in %d files", (long)(textsize/1024), nfile);
     if (rev_mode == ExecuteExport)
 	export_init();
     load_total_files = nfile;
     load_current_file = 0;
+    /* analyze the files for CVS revision structure */
     while (fn_head) {
 	int nversions;
 	
@@ -530,34 +622,34 @@ main (int argc, char **argv)
 	fn_head = fn_head->next;
 	++load_current_file;
 	if (verbose)
-	    fprintf(stderr, "Processing %s\n", fn->file);
-	load_status (fn->file + strip);
+	    announce("processing %s\n", fn->file);
+	if (progress)
+	    load_status (fn->file + strip);
 	rl = rev_list_file (fn->file, &nversions);
-	if (rl->watch)
-	    dump_rev_tree (rl);
 	*tail = rl;
 	tail = &rl->next;
 
 	free(fn);
     }
-    if (skew_vulnerable > 0 && load_total_files > 1)
-	fprintf(stderr, "Commits before this date lack commitids: %s",
-		ctime(&skew_vulnerable));
-    load_status_next ();
+    if (progress)
+	load_status_next ();
+    /* commit set coalescence happens here */
     rl = rev_list_merge (head);
+    /* report on the DAG */
     if (rl) {
 	switch (rev_mode) {
 	case ExecuteGraph:
 	    dump_rev_graph (rl, NULL);
 	    break;
-	case ExecuteSplits:
-	    dump_splits (rl);
-	    break;
 	case ExecuteExport:
-	    export_commits (rl, strip);
+	    export_commits (rl, strip, fromtime, progress);
 	    break;
 	}
     }
+    if (skew_vulnerable > 0 && load_total_files > 1 && !force_dates) {
+	time_t udate = RCS_EPOCH + skew_vulnerable;
+	announce("commits before this date lack commitids: %s",	ctime(&udate));
+    }
     if (rl)
 	rev_list_free (rl, 0);
     while (head) {
@@ -568,9 +660,12 @@ main (int argc, char **argv)
     discard_atoms ();
     discard_tags ();
     rev_free_dirs ();
-    rev_commit_cleanup ();
+    git_commit_cleanup ();
+    export_wrap();
     free_author_map ();
     if (revision_map)
 	fclose(revision_map);
     return err;
 }
+
+/* end */
