@@ -24,11 +24,11 @@
  * Given a single-file tree, locate the specific version number
  */
 
-static rev_commit *
-rev_find_cvs_commit (rev_list *rl, cvs_number *number)
+static cvs_commit *
+rev_find_cvs_revision (rev_list *rl, cvs_number *number)
 {
     rev_ref	*h;
-    rev_commit	*c;
+    cvs_commit	*c;
     rev_file	*f;
 
     for (h = rl->heads; h; h = h->next) {
@@ -49,32 +49,29 @@ rev_find_cvs_commit (rev_list *rl, cvs_number *number)
 /*
  * Construct a branch using CVS revision numbers
  */
-static rev_commit *
+static cvs_commit *
 rev_branch_cvs (cvs_file *cvs, cvs_number *branch)
 {
     cvs_number	n;
-    rev_commit	*head = NULL;
-    rev_commit	*c, *p, *gc;
+    cvs_commit	*head = NULL;
+    cvs_commit	*c, *p, *gc;
     Node	*node;
 
     n = *branch;
     n.n[n.c-1] = -1;
     for (node = cvs_find_version (cvs, &n); node; node = node->next) {
-	cvs_version *v = node->v;
-	cvs_patch *p = node->p;
-	rev_commit *c;
+	cvs_version *v = node->version;
+	cvs_patch *p = node->patch;
+	cvs_commit *c;
 	if (!v)
 	     continue;
-	c = calloc (1, sizeof (rev_commit));
+	c = xcalloc (1, sizeof (cvs_commit), "branch construction");
 	c->date = v->date;
 	c->commitid = v->commitid;
 	c->author = v->author;
 	if (p)
 	    c->log = p->log;
-	if (v->dead)
-	    c->nfiles = 0;
-	else
-	    c->nfiles = 1;
+	 c->dead = v->dead;
 	/* leave this around so the branch merging stuff can find numbers */
 	c->file = rev_file_rev (cvs->name, &v->number, v->date);
 	if (!v->dead) {
@@ -85,15 +82,18 @@ rev_branch_cvs (cvs_file *cvs, cvs_number *branch)
 	head = c;
     }
 
+	if (head == NULL)
+		return NULL;
+
     /*
      * Make sure the dates along the branch are well ordered. As we
      * want to preserve current data, push previous versions back to
      * align with newer revisions.
      */
     for (c = head, gc = NULL; (p = c->parent); gc = c, c = p) {
-	if (time_compare (p->file->date, c->file->date) > 0)
+	if (time_compare (p->file->u.date, c->file->u.date) > 0)
 	{
-	    fprintf (stderr, "Warning: %s:", cvs->name);
+	    fprintf (stderr, "cvs-fast-export: warning - %s:", cvs->name);
 	    dump_number_file (stderr, " ", &p->file->number);
 	    dump_number_file (stderr, " is newer than", &c->file->number);
 
@@ -101,14 +101,14 @@ rev_branch_cvs (cvs_file *cvs, cvs_number *branch)
 	     * clock set wrong.  Dont push back all commits for that,
 	     * just fix up the current commit instead of the
 	     * parent. */
-	    if (gc && time_compare (p->file->date, gc->file->date) <= 0)
+	    if (gc && time_compare (p->file->u.date, gc->file->u.date) <= 0)
 	    {
 	      dump_number_file (stderr, ", adjusting", &c->file->number);
-	      c->file->date = p->file->date;
+	      c->file->u.date = p->file->u.date;
 	      c->date = p->date;
 	    } else {
 	      dump_number_file (stderr, ", adjusting", &c->file->number);
-	      p->file->date = c->file->date;
+	      p->file->u.date = c->file->u.date;
 	      p->date = c->date;
 	    }
 	    fprintf (stderr, "\n");
@@ -135,14 +135,13 @@ rev_list_patch_vendor_branch (rev_list *rl, cvs_file *cvs)
     rev_ref	*trunk = NULL;
     rev_ref	*vendor = NULL;
     rev_ref	*h;
-    rev_commit	*t, **tp, *v, **vp;
-    rev_commit	*vlast;
+    cvs_commit	*t, **tp, *v, **vp;
+    cvs_commit	*vlast;
     rev_ref	**h_p;
-    int		delete_head;
 
     trunk = rl->heads;
     for (h_p = &rl->heads; (h = *h_p);) {
-	delete_head = 0;
+	int delete_head = 0;
 	if (h->commit && cvs_is_vendor (&h->commit->file->number))
 	{
 	    /*
@@ -166,8 +165,8 @@ rev_list_patch_vendor_branch (rev_list *rl, cvs_file *cvs)
 	    while ((t = *tp))
 	    {
 		if (!t->parent || 
-		    time_compare (vlast->file->date,
-				  t->parent->file->date) >= 0)
+		    time_compare (vlast->file->u.date,
+				  t->parent->file->u.date) >= 0)
 		{
 		    break;
 		}
@@ -180,8 +179,8 @@ rev_list_patch_vendor_branch (rev_list *rl, cvs_file *cvs)
 		 * of the vendor branch, paste them together and
 		 * nuke the vendor branch
 		 */
-		if (time_compare (vlast->file->date,
-				  t->file->date) >= 0)
+		if (time_compare (vlast->file->u.date,
+				  t->file->u.date) >= 0)
 		{
 		    delete_head = 1;
 		}
@@ -217,7 +216,7 @@ rev_list_patch_vendor_branch (rev_list *rl, cvs_file *cvs)
 	     * Patch up the remaining vendor branch pieces
 	     */
 	    if (!delete_head) {
-		rev_commit  *vr;
+		cvs_commit  *vr;
 		if (!vendor->name) {
 		    char	rev[CVS_MAX_REV_LEN];
 		    char	name[MAXPATHLEN];
@@ -235,7 +234,7 @@ rev_list_patch_vendor_branch (rev_list *rl, cvs_file *cvs)
 		for (vr = vendor->commit; vr; vr = vr->parent)
 		{
 		    if (!vr->parent) {
-			vr->tail = 1;
+			vr->tail = true;
 			vr->parent = v;
 			break;
 		    }
@@ -247,8 +246,8 @@ rev_list_patch_vendor_branch (rev_list *rl, cvs_file *cvs)
 	     */
 	    while (t && v)
 	    {
-		if (time_compare (v->file->date,
-				  t->file->date) >= 0)
+		if (time_compare (v->file->u.date,
+				  t->file->u.date) >= 0)
 		{
 		    *tp = v;
 		    tp = &v->parent;
@@ -291,7 +290,7 @@ static void
 rev_list_graft_branches (rev_list *rl, cvs_file *cvs)
 {
     rev_ref	*h;
-    rev_commit	*c;
+    cvs_commit	*c;
     cvs_version	*cv;
     cvs_branch	*cb;
 
@@ -328,8 +327,8 @@ rev_list_graft_branches (rev_list *rl, cvs_file *cvs)
 		    if (cvs_number_compare (&cb->number,
 					    &c->file->number) == 0)
 		    {
-			c->parent = rev_find_cvs_commit (rl, &cv->number);
-			c->tail = 1;
+			c->parent = rev_find_cvs_revision (rl, &cv->number);
+			c->tail = true;
 			break;
 		    }
 		}
@@ -342,14 +341,14 @@ rev_list_graft_branches (rev_list *rl, cvs_file *cvs)
 		    for (cb = cv->branches; cb; cb = cb->next) {
 			if (cvs_is_vendor (&cb->number)) {
 			    cvs_number	v_n;
-			    rev_commit	*v_c, *n_v_c;
+			    cvs_commit	*v_c, *n_v_c;
 			    fprintf (stderr, "Found merge into vendor branch\n");
 			    v_n = cb->number;
 			    v_c = NULL;
 			    /*
 			     * Walk to head of vendor branch
 			     */
-			    while ((n_v_c = rev_find_cvs_commit (rl, &v_n)))
+			    while ((n_v_c = rev_find_cvs_revision (rl, &v_n)))
 			    {
 				/*
 				 * Stop if we reach a date after the
@@ -413,13 +412,12 @@ rev_list_set_refs (rev_list *rl, cvs_file *cvs)
 {
     rev_ref	*h;
     cvs_symbol	*s;
-    rev_commit	*c;
     
     /*
      * Locate a symbolic name for this head
      */
     for (s = cvs->symbols; s; s = s->next) {
-	c = NULL;
+	cvs_commit	*c = NULL;
 	if (cvs_is_head (&s->number)) {
 	    for (h = rl->heads; h; h = h->next) {
 		if (cvs_same_branch (&h->commit->file->number, &s->number))
@@ -438,7 +436,7 @@ rev_list_set_refs (rev_list *rl, cvs_file *cvs)
 		n = s->number;
 		while (n.c >= 4) {
 		    n.c -= 2;
-		    c = rev_find_cvs_commit (rl, &n);
+		    c = rev_find_cvs_revision (rl, &n);
 		    if (c)
 			break;
 		}
@@ -449,7 +447,7 @@ rev_list_set_refs (rev_list *rl, cvs_file *cvs)
 	    if (h)
 		h->number = s->number;
 	} else {
-	    c = rev_find_cvs_commit (rl, &s->number);
+	    c = rev_find_cvs_revision (rl, &s->number);
 	    if (c)
 		tag_commit(c, s->name);
 	}
@@ -459,12 +457,12 @@ rev_list_set_refs (rev_list *rl, cvs_file *cvs)
      */
     for (h = rl->heads; h; h = h->next) {
 	cvs_number	n;
-	rev_commit	*c;
+	cvs_commit	*c;
 
 	if (h->name)
 	    continue;
 	for (c = h->commit; c; c = c->parent) {
-	    if (c->nfiles)
+	    if (!c->dead)
 		break;
 	}
 	if (!c)
@@ -488,7 +486,7 @@ rev_list_set_refs (rev_list *rl, cvs_file *cvs)
 	    n.c -= 2;
 	    h->parent = rev_list_find_branch (rl, &n);
 	    if (!h->parent && ! cvs_is_vendor (&h->number))
-		fprintf (stderr, "Warning: %s: branch %s has no parent\n",
+		announce("warning - %s branch %s has no parent\n",
 			 cvs->name, h->name);
 	}
 	if (h->parent && !h->name) {
@@ -496,9 +494,9 @@ rev_list_set_refs (rev_list *rl, cvs_file *cvs)
 	    char	rev[CVS_MAX_REV_LEN];
 
 	    cvs_number_string (&h->number, rev);
-	    fprintf (stderr, "Warning: %s: unnamed branch %s from %s\n",
-		     cvs->name, rev, h->parent->name);
-	    sprintf (name, "%s-UNNAMED-BRANCH", h->parent->name);
+	    sprintf (name, "%s-UNNAMED-BRANCH-%s", h->parent->name, h->commit->commitid);
+	    announce("warning - putting %s rev %s on unnamed branch %s off %s\n",
+		     cvs->name, rev, name, h->parent->name);
 	    h->name = atom (name);
 	}
     }
@@ -514,13 +512,13 @@ static void
 rev_list_free_dead_files (rev_list *rl)
 {
     rev_ref	*h;
-    rev_commit	*c;
+    cvs_commit	*c;
 
     for (h = rl->heads; h; h = h->next) {
 	if (h->tail)
 	    continue;
 	for (c = h->commit; c; c = c->parent) {
-	    if (c->nfiles == 0) {
+	    if (c->dead) {
 		rev_file_free (c->file);
 		c->file = 0;
 	    }
@@ -530,7 +528,7 @@ rev_list_free_dead_files (rev_list *rl)
     }
 }
 
-#if UNUSED
+#ifdef __UNUSED__
 static int
 rev_order_compare (cvs_number *a, cvs_number *b)
 {
@@ -540,14 +538,32 @@ rev_order_compare (cvs_number *a, cvs_number *b)
 }
 #endif
 
+static int
+cvs_symbol_name_compare (void *x, void *y)
+{
+    if (x < y)
+	return -1;
+    else if (y < x)
+	return 1;
+    else
+	return 0;
+}
+
 static cvs_symbol *
 cvs_find_symbol (cvs_file *cvs, char *name)
 {
-    cvs_symbol	*s;
+    rbtree_node *n, **tree;
 
-    for (s = cvs->symbols; s; s = s->next)
-	if (s->name == name)
-	    return s;
+    tree = &cvs->symbols_by_name;
+    if (!(*tree)) {
+	cvs_symbol *s;
+	for (s = cvs->symbols; s ; s = s->next)
+	    rbtree_insert (tree, s->name, s, cvs_symbol_name_compare);
+    }
+
+    n = rbtree_lookup (*tree, name, cvs_symbol_name_compare);
+    if (n)
+	return (cvs_symbol*)n->value;
     return NULL;
 }
 
@@ -567,9 +583,9 @@ static void
 rev_list_sort_heads (rev_list *rl, cvs_file *cvs)
 {
     rev_ref	*h, **hp;
-    cvs_symbol	*hs, *hns;
 
     for (hp = &rl->heads; (h = *hp);) {
+	cvs_symbol	*hs, *hns;
 	if (!h->next)
 	    break;
 	hs = cvs_find_symbol (cvs, h->name);
@@ -598,13 +614,12 @@ rev_list_sort_heads (rev_list *rl, cvs_file *cvs)
 rev_list *
 rev_list_cvs (cvs_file *cvs)
 {
-    rev_list	*rl = calloc (1, sizeof (rev_list));
+    rev_list	*rl = xcalloc (1, sizeof (rev_list), "rev_list_cvs");
     cvs_number	trunk_number;
-    rev_commit	*trunk; 
-    rev_commit	*branch;
+    cvs_commit	*trunk; 
+    cvs_commit	*branch;
     cvs_version	*cv;
     cvs_branch	*cb;
-    rev_ref	*t;
     cvs_version	*ctrunk = NULL;
 
     build_branches();
@@ -628,11 +643,12 @@ rev_list_cvs (cvs_file *cvs)
 	trunk_number = lex_number ("1.1");
     trunk = rev_branch_cvs (cvs, &trunk_number);
     if (trunk) {
+	rev_ref	*t;
 	t = rev_list_add_head (rl, trunk, atom ("master"), 2);
 	t->number = trunk_number;
     }
     else
-	fprintf(stderr, "warning - no master branch generated\n");
+	announce("warning - no master branch generated\n");
     /*
      * Search for other branches
      */
