@@ -19,18 +19,20 @@
 #include "cvs.h"
 
 /*
- * Add head refs
+ * A revision list is the history for an entire RCS/CVS repository.
+ * These functions analyze a revlist into a DAG.
  */
 
 rev_ref *
-rev_list_add_head (rev_list *rl, rev_commit *commit, char *name, int degree)
+rev_list_add_head (rev_list *rl, cvs_commit *commit, char *name, int degree)
+/* Add head refs */
 {
     rev_ref	*r;
     rev_ref	**list = &rl->heads;
 
     while (*list)
 	list = &(*list)->next;
-    r = calloc (1, sizeof (rev_ref));
+    r = xcalloc (1, sizeof (rev_ref), "adding head reference");
     r->commit = commit;
     r->name = name;
     r->next = *list;
@@ -56,6 +58,7 @@ rev_find_head (rev_list *rl, char *name)
  * (which are always unique)
  */
 
+#ifdef __UNUSED__
 bool
 rev_file_later (rev_file *af, rev_file *bf)
 {
@@ -79,7 +82,7 @@ rev_file_later (rev_file *af, rev_file *bf)
 }
 
 bool
-rev_commit_later (rev_commit *a, rev_commit *b)
+git_commit_later (git_commit *a, git_commit *b)
 {
     long	t;
 
@@ -93,9 +96,11 @@ rev_commit_later (rev_commit *a, rev_commit *b)
 	return true;
     return true;
 }
+#endif /* __UNUSED__ */
 
 static bool
-commit_time_close (time_t a, time_t b)
+commit_time_close (cvstime_t a, cvstime_t b)
+/* are two timestamps witin the commit-coalescence window? */
 {
     long	diff = a - b;
     if (diff < 0) diff = -diff;
@@ -104,15 +109,12 @@ commit_time_close (time_t a, time_t b)
     return false;
 }
 
-/*
- * The heart of the merge operation; detect when two
- * commits are "the same"
- */
 static bool
-rev_commit_match (rev_commit *a, rev_commit *b)
+cvs_commit_match (cvs_commit *a, cvs_commit *b)
+/* are two commits eligible to be coalesced into a changeset? */
 {
     /*
-     * Very recent versions of CVS place a commitid in
+     * Versions of GNU CVS after 1.12 (2004) place a commitid in
      * each commit to track patch sets. Use it if present
      */
     if (a->commitid && b->commitid)
@@ -128,9 +130,9 @@ rev_commit_match (rev_commit *a, rev_commit *b)
     return true;
 }
 
-#if UNUSED
+#ifdef __UNUSED__
 static void
-rev_commit_dump (FILE *f, char *title, rev_commit *c, rev_commit *m)
+git_commit_dump (FILE *f, char *title, git_commit *c, git_commit *m)
 {
     fprintf (f, "\n%s\n", title);
     while (c) {
@@ -153,19 +155,18 @@ void
 rev_list_set_tail (rev_list *rl)
 {
     rev_ref	*head;
-    rev_commit	*c;
-    int		tail;
+    cvs_commit	*c;
 
     for (head = rl->heads; head; head = head->next) {
-	tail = 1;
+	flag tail = true;
 	if (head->commit && head->commit->seen) {
 	    head->tail = tail;
-	    tail = 0;
+	    tail = false;
 	}
 	for (c = head->commit; c; c = c->parent) {
 	    if (tail && c->parent && c->seen < c->parent->seen) {
-		c->tail = 1;
-		tail = 0;
+		c->tail = true;
+		tail = false;
 	    }
 	    c->seen++;
 	}
@@ -259,17 +260,17 @@ rev_ref_find_name (rev_ref *h, char *name)
     return NULL;
 }
 
-static int
+static bool
 rev_ref_is_ready (char *name, rev_list *source, rev_ref *ready)
 {
     for (; source; source = source->next) {
 	rev_ref *head = rev_find_head(source, name);
 	if (head) {
 	    if (head->parent && !rev_ref_find_name(ready, head->parent->name))
-		    return 0;
+		    return false;
 	}
     }
-    return 1;
+    return true;
 }
 
 static rev_ref *
@@ -287,7 +288,7 @@ rev_ref_tsort (rev_ref *refs, rev_list *head)
 	    }
 	}
 	if (!r) {
-	    fprintf (stderr, "Error: branch cycle\n");
+	    announce("internal error - branch cycle\n");
 	    return NULL;
 	}
 	*prev = r->next;
@@ -311,10 +312,10 @@ rev_list_count (rev_list *head)
 }
 
 static int
-rev_commit_date_compare (const void *av, const void *bv)
+cvs_commit_date_compare (const void *av, const void *bv)
 {
-    const rev_commit	*a = *(const rev_commit **) av;
-    const rev_commit	*b = *(const rev_commit **) bv;
+    const cvs_commit	*a = *(const cvs_commit **) av;
+    const cvs_commit	*b = *(const cvs_commit **) bv;
     int			t;
 
     /*
@@ -355,10 +356,10 @@ rev_commit_date_compare (const void *av, const void *bv)
 }
 
 static int
-rev_commit_date_sort (rev_commit **commits, int ncommit)
+cvs_commit_date_sort (cvs_commit **commits, int ncommit)
 {
-    qsort (commits, ncommit, sizeof (rev_commit *),
-	   rev_commit_date_compare);
+    qsort (commits, ncommit, sizeof (cvs_commit *),
+	   cvs_commit_date_compare);
     /*
      * Trim off NULL entries
      */
@@ -367,25 +368,25 @@ rev_commit_date_sort (rev_commit **commits, int ncommit)
     return ncommit;
 }
 
-int
-rev_commit_has_file (rev_commit *c, rev_file *f)
+bool
+git_commit_has_file (git_commit *c, rev_file *f)
 {
     int	i, j;
 
     if (!c)
-	return 0;
+	return false;
     for (i = 0; i < c->ndirs; i++) {
 	rev_dir	*dir = c->dirs[i];
 	for (j = 0; j < dir->nfiles; j++)
 	    if (dir->files[j] == f)
-		return 1;
+		return true;
     }
-    return 0;
+    return false;
 }
 
-#if UNUSED
+#ifdef __UNUSED__
 static rev_file *
-rev_commit_find_file (rev_commit *c, char *name)
+git_commit_find_file (git_commit *c, char *name)
 {
     int	n;
 
@@ -400,7 +401,7 @@ static rev_file **files = NULL;
 static int	    sfiles = 0;
 
 void
-rev_commit_cleanup (void)
+git_commit_cleanup (void)
 {
     if (files) {
 	free (files);
@@ -409,26 +410,28 @@ rev_commit_cleanup (void)
     }
 }
 
-static rev_commit *
-rev_commit_build (rev_commit **commits, rev_commit *leader, int ncommit)
+static git_commit *
+git_commit_build (cvs_commit **revisions, cvs_commit *leader, int nrevisions)
+/* build a changeset commit from a clique of CVS revisions */
 {
     int		n, nfile;
-    rev_commit	*commit;
+    git_commit	*commit;
     int		nds;
     rev_dir	**rds;
     rev_file	*first;
 
-    if (ncommit > sfiles) {
+    if (nrevisions > sfiles) {
 	free (files);
 	files = 0;
     }
     if (!files)
-	files = malloc ((sfiles = ncommit) * sizeof (rev_file *));
+	/* coverity[sizecheck] Coverity has a bug here */
+	files = xmalloc ((sfiles = nrevisions) * sizeof (rev_file *), __func__);
     
     nfile = 0;
-    for (n = 0; n < ncommit; n++)
-	if (commits[n] && commits[n]->file)
-	    files[nfile++] = commits[n]->file;
+    for (n = 0; n < nrevisions; n++)
+	if (revisions[n] && revisions[n]->file)
+	    files[nfile++] = revisions[n]->file;
     
     if (nfile)
 	first = files[0];
@@ -437,8 +440,8 @@ rev_commit_build (rev_commit **commits, rev_commit *leader, int ncommit)
     
     rds = rev_pack_files (files, nfile, &nds);
         
-    commit = calloc (1, sizeof (rev_commit) +
-		     nds * sizeof (rev_dir *));
+    commit = xcalloc (1, sizeof (git_commit) +
+		      nds * sizeof (rev_dir *), "creating commit");
     
     commit->date = leader->date;
     commit->commitid = leader->commitid;
@@ -453,36 +456,37 @@ rev_commit_build (rev_commit **commits, rev_commit *leader, int ncommit)
     return commit;
 }
 
-#if UNUSED
-static rev_commit *
+#ifdef __UNUSED__
+static git_commit *
 rev_ref_find_commit_file (rev_ref *branch, rev_file *file)
 {
-    rev_commit	*c;
+    git_commit	*c;
 
     for (c = branch->commit; c; c = c->parent)
-	if (rev_commit_has_file (c, file))
+	if (git_commit_has_file (c, file))
 	    return c;
     return NULL;
 }
 
-static int
-rev_commit_is_ancestor (rev_commit *old, rev_commit *young)
+static bool
+git_commit_is_ancestor (git_commit *old, git_commit *young)
 {
     while (young) {
 	if (young == old)
-	    return 1;
+	    return true;
 	young = young->parent;
     }
-    return 0;
+    return false;
 }
 #endif
 
-static rev_commit *
-rev_commit_locate_date (rev_ref *branch, time_t date)
+static git_commit *
+git_commit_locate_date (rev_ref *branch, cvstime_t date)
 {
-    rev_commit	*commit;
+    git_commit	*commit;
 
-    for (commit = branch->commit; commit; commit = commit->parent)
+    /* PUNNING: see the big comment in cvs.h */ 
+    for (commit = (git_commit *)branch->commit; commit; commit = commit->parent)
     {
 	if (time_compare (commit->date, date) <= 0)
 	    return commit;
@@ -490,46 +494,48 @@ rev_commit_locate_date (rev_ref *branch, time_t date)
     return NULL;
 }
 
-static rev_commit *
-rev_commit_locate_one (rev_ref *branch, rev_commit *file)
+static git_commit *
+git_commit_locate_one (rev_ref *branch, cvs_commit *file)
 {
-    rev_commit	*commit;
+    git_commit	*commit;
 
     if (!branch)
 	return NULL;
 
-    for (commit = branch->commit;
+    /* PUNNING: see the big comment in cvs.h */ 
+    for (commit = (git_commit *)branch->commit;
 	 commit;
 	 commit = commit->parent)
     {
-	if (rev_commit_match (commit, file))
+	/* PUNNING: see the big comment in cvs.h */ 
+	if (cvs_commit_match ((cvs_commit *)commit, file))
 	    return commit;
     }
     return NULL;
 }
 
-static rev_commit *
-rev_commit_locate_any (rev_ref *branch, rev_commit *file)
+static git_commit *
+git_commit_locate_any (rev_ref *branch, cvs_commit *file)
 {
-    rev_commit	*commit;
+    git_commit	*commit;
 
     if (!branch)
 	return NULL;
-    commit = rev_commit_locate_any (branch->next, file);
+    commit = git_commit_locate_any (branch->next, file);
     if (commit)
 	return commit;
-    return rev_commit_locate_one (branch, file);
+    return git_commit_locate_one (branch, file);
 }
 
-static rev_commit *
-rev_commit_locate (rev_ref *branch, rev_commit *file)
+static git_commit *
+git_commit_locate (rev_ref *branch, cvs_commit *file)
 {
-    rev_commit	*commit;
+    git_commit	*commit;
 
     /*
      * Check the presumed trunk first
      */
-    commit = rev_commit_locate_one (branch, file);
+    commit = git_commit_locate_one (branch, file);
     if (commit)
 	return commit;
     /*
@@ -537,21 +543,21 @@ rev_commit_locate (rev_ref *branch, rev_commit *file)
      */
     while (branch->parent)
 	branch = branch->parent;
-    return rev_commit_locate_any (branch, file);
+    return git_commit_locate_any (branch, file);
 }
 
 rev_ref *
-rev_branch_of_commit (rev_list *rl, rev_commit *commit)
+rev_branch_of_commit (rev_list *rl, cvs_commit *commit)
 {
     rev_ref	*h;
-    rev_commit	*c;
+    cvs_commit	*c;
 
     for (h = rl->heads; h; h = h->next)
     {
 	if (h->tail)
 	    continue;
 	for (c = h->commit; c; c = c->parent) {
-	    if (rev_commit_match (c, commit))
+	    if (cvs_commit_match (c, commit))
 		return h;
 	    if (c->tail)
 		break;
@@ -563,8 +569,8 @@ rev_branch_of_commit (rev_list *rl, rev_commit *commit)
 /*
  * Time of first commit along entire history
  */
-static time_t
-rev_commit_first_date (rev_commit *commit)
+static cvstime_t
+cvs_commit_first_date (cvs_commit *commit)
 {
     while (commit->parent)
 	commit = commit->parent;
@@ -578,249 +584,250 @@ static void
 rev_branch_merge (rev_ref **branches, int nbranch,
 		  rev_ref *branch, rev_list *rl)
 {
-	int nlive;
-	int n;
-	rev_commit *prev = NULL;
-	rev_commit *head = NULL, **tail = &head;
-	rev_commit **commits = calloc (nbranch, sizeof (rev_commit *));
-	rev_commit *commit;
-	rev_commit *latest;
-	rev_commit **p;
-	time_t start = 0;
+    int nlive;
+    int n;
+    git_commit *prev = NULL;
+    git_commit *head = NULL, **tail = &head;
+    cvs_commit **revisions = xcalloc (nbranch, sizeof (cvs_commit *), "merging per-file branches");
+    git_commit *commit;
+    cvs_commit *latest;
+    cvs_commit **p;
+    time_t start = 0;
 
+    nlive = 0;
+    for (n = 0; n < nbranch; n++) {
+	cvs_commit *c;
+	/*
+	 * Initialize revisions to head of each branch
+	 */
+	c = revisions[n] = branches[n]->commit;
+	/*
+	 * Compute number of branches with remaining entries
+	 */
+	if (!c)
+	    continue;
+	if (branches[n]->tail) {
+	    c->tailed = true;
+	    continue;
+	}
+	nlive++;
+	while (c && !c->tail) {
+	    if (!start || time_compare(c->date, start) < 0)
+		start = c->date;
+	    c = c->parent;
+	}
+	if (c && (c->file || c->date != c->parent->date)) {
+	    if (!start || time_compare(c->date, start) < 0)
+		start = c->date;
+	}
+    }
+
+    for (n = 0; n < nbranch; n++) {
+	cvs_commit *c = revisions[n];
+	if (!c->tailed)
+	    continue;
+	if (!start || time_compare(start, c->date) >= 0)
+	    continue;
+	if (c->file)
+	    announce("warning - %s too late date through branch %s\n",
+		     c->file->name, branch->name);
+	revisions[n] = NULL;
+    }
+    /*
+     * Walk down branches until each one has merged with the
+     * parent branch
+     */
+    while (nlive > 0 && nbranch > 0) {
+	for (n = 0, p = revisions, latest = NULL; n < nbranch; n++) {
+	    cvs_commit *rev = revisions[n];
+	    if (!rev)
+		continue;
+	    *p++ = rev;
+	    if (rev->tailed)
+		continue;
+	    if (!latest || time_compare(latest->date, rev->date) < 0)
+		latest = rev;
+	}
+	nbranch = p - revisions;
+
+	/*
+	 * Construct current commit
+	 */
+	commit = git_commit_build (revisions, latest, nbranch);
+
+	/*
+	 * Step each branch
+	 */
 	nlive = 0;
 	for (n = 0; n < nbranch; n++) {
-		rev_commit *c;
+	    cvs_commit *c = revisions[n];
+	    cvs_commit *to;
+	    /* already got to parent branch? */
+	    if (c->tailed)
+		continue;
+	    /* not affected? */
+	    if (c != latest && !cvs_commit_match(c, latest)) {
+		if (c->parent || c->file)
+		    nlive++;
+		continue;
+	    }
+	    to = c->parent;
+	    /* starts here? */
+	    if (!to)
+		goto Kill;
+
+	    if (c->tail) {
 		/*
-		 * Initialize commits to head of each branch
+		 * Adding file independently added on another
+		 * non-trunk branch.
 		 */
-		c = commits[n] = branches[n]->commit;
+		if (!to->parent && !to->file)
+		    goto Kill;
 		/*
-		* Compute number of branches with remaining entries
-		*/
-		if (!c)
-			continue;
-		if (branches[n]->tail) {
-			c->tailed = true;
-			continue;
-		}
+		 * If the parent is at the beginning of trunk
+		 * and it is younger than some events on our
+		 * branch, we have old CVS adding file
+		 * independently
+		 * added on another branch.
+		 */
+		if (start && time_compare(start, to->date) < 0)
+		    goto Kill;
+		/*
+		 * XXX: we still can't be sure that it's
+		 * not a file added on trunk after parent
+		 * branch had forked off it but before
+		 * our branch's creation.
+		 */
+		to->tailed = true;
+	    } else if (to->file) {
 		nlive++;
-		while (c && !c->tail) {
-			if (!start || time_compare(c->date, start) < 0)
-				start = c->date;
-			c = c->parent;
-		}
-		if (c && (c->file || c->date != c->parent->date)) {
-			if (!start || time_compare(c->date, start) < 0)
-				start = c->date;
-		}
-	}
-
-	for (n = 0; n < nbranch; n++) {
-		rev_commit *c = commits[n];
-		if (!c->tailed)
-			continue;
-		if (!start || time_compare(start, c->date) >= 0)
-			continue;
-		if (c->file)
-			fprintf(stderr,
-				"Warning: %s too late date through branch %s\n",
-					c->file->name, branch->name);
-		commits[n] = NULL;
-	}
-	/*
-	 * Walk down branches until each one has merged with the
-	 * parent branch
-	 */
-	while (nlive > 0 && nbranch > 0) {
-		for (n = 0, p = commits, latest = NULL; n < nbranch; n++) {
-			rev_commit *c = commits[n];
-			if (!c)
-				continue;
-			*p++ = c;
-			if (c->tailed)
-				continue;
-			if (!latest || time_compare(latest->date, c->date) < 0)
-				latest = c;
-		}
-		nbranch = p - commits;
-
+	    } else {
 		/*
-		 * Construct current commit
+		 * See if it's recent CVS adding a file
+		 * independently added on another branch.
 		 */
-		commit = rev_commit_build (commits, latest, nbranch);
-
-		/*
-		 * Step each branch
-		 */
-		nlive = 0;
-		for (n = 0; n < nbranch; n++) {
-			rev_commit *c = commits[n];
-			rev_commit *to;
-			/* already got to parent branch? */
-			if (c->tailed)
-				continue;
-			/* not affected? */
-			if (c != latest && !rev_commit_match(c, latest)) {
-				if (c->parent || c->file)
-					nlive++;
-				continue;
-			}
-			to = c->parent;
-			/* starts here? */
-			if (!to)
-				goto Kill;
-
-			if (c->tail) {
-				/*
-				 * Adding file independently added on another
-				 * non-trunk branch.
-				 */
-				if (!to->parent && !to->file)
-					goto Kill;
-				/*
-				 * If the parent is at the beginning of trunk
-				 * and it is younger than some events on our
-				 * branch, we have old CVS adding file
-				 * independently
-				 * added on another branch.
-				 */
-				if (start && time_compare(start, to->date) < 0)
-					goto Kill;
-				/*
-				 * XXX: we still can't be sure that it's
-				 * not a file added on trunk after parent
-				 * branch had forked off it but before
-				 * our branch's creation.
-				 */
-				to->tailed = true;
-			} else if (to->file) {
-				nlive++;
-			} else {
-				/*
-				 * See if it's recent CVS adding a file
-				 * independently added on another branch.
-				 */
-				if (!to->parent)
-					goto Kill;
-				if (to->tail && to->date == to->parent->date)
-					goto Kill;
-				nlive++;
-			}
-			commits[n] = to;
-			continue;
-Kill:
-			commits[n] = NULL;
-		}
-
-		*tail = commit;
-		tail = &commit->parent;
-		prev = commit;
+		if (!to->parent)
+		    goto Kill;
+		if (to->tail && to->date == to->parent->date)
+		    goto Kill;
+		nlive++;
+	    }
+	    revisions[n] = to;
+	    continue;
+	Kill:
+	    revisions[n] = NULL;
 	}
+
+	*tail = commit;
+	tail = &commit->parent;
+	prev = commit;
+    }
     /*
      * Connect to parent branch
      */
-    nbranch = rev_commit_date_sort (commits, nbranch);
+    nbranch = cvs_commit_date_sort (revisions, nbranch);
     if (nbranch && branch->parent )
     {
-	rev_ref	*lost;
 	int	present;
 
 //	present = 0;
 	for (present = 0; present < nbranch; present++)
-	    if (commits[present]->file) {
+	    if (revisions[present]->file) {
 		/*
 		 * Skip files which appear in the repository after
 		 * the first commit along the branch
 		 */
-		if (prev && commits[present]->date > prev->date &&
-		    commits[present]->date == rev_commit_first_date (commits[present]))
+		if (prev && revisions[present]->date > prev->date &&
+		    revisions[present]->date == cvs_commit_first_date (revisions[present]))
 		{
-		    fprintf (stderr, "Warning: file %s appears after branch %s date\n",
-			     commits[present]->file->name, branch->name);
+		    /* FIXME: what does this mean? */
+		    announce("warning - file %s appears after branch %s date\n",
+			     revisions[present]->file->name, branch->name);
 		    continue;
 		}
 		break;
 	    }
 	if (present == nbranch)
 	    *tail = NULL;
-	else if ((*tail = rev_commit_locate_one (branch->parent,
-						 commits[present])))
+	else if ((*tail = git_commit_locate_one (branch->parent,
+						 revisions[present])))
 	{
 	    if (prev && time_compare ((*tail)->date, prev->date) > 0) {
-		fprintf (stderr, "Warning: branch point %s -> %s later than branch\n",
+		announce("warning - branch point %s -> %s later than branch\n",
 			 branch->name, branch->parent->name);
 		fprintf (stderr, "\ttrunk(%3d):  %s %s", n,
-			 ctime_nonl (&commits[present]->date),
-			 commits[present]->file ? " " : "D" );
-		if (commits[present]->file)
+			 ctime_nonl (&revisions[present]->date),
+			 revisions[present]->file ? " " : "D" );
+		if (revisions[present]->file)
 		    dump_number_file (stderr,
-				      commits[present]->file->name,
-				      &commits[present]->file->number);
+				      revisions[present]->file->name,
+				      &revisions[present]->file->number);
 		fprintf (stderr, "\n");
 		fprintf (stderr, "\tbranch(%3d): %s  ", n,
-			 ctime_nonl (&prev->file->date));
+			 ctime_nonl (&prev->file->u.date));
 		dump_number_file (stderr,
 				  prev->file->name,
 				  &prev->file->number);
 		fprintf (stderr, "\n");
 	    }
-	} else if ((*tail = rev_commit_locate_date (branch->parent,
-						  commits[present]->date)))
-	    fprintf (stderr, "Warning: branch point %s -> %s matched by date\n",
+	} else if ((*tail = git_commit_locate_date (branch->parent,
+						    revisions[present]->date)))
+	    announce("warning - branch point %s -> %s matched by date\n",
 		     branch->name, branch->parent->name);
 	else {
-	    fprintf (stderr, "Error: branch point %s -> %s not found.",
-		     branch->name, branch->parent->name);
+	    rev_ref	*lost;
+	    fprintf(stderr, "Error: branch point %s -> %s not found.",
+		    branch->name, branch->parent->name);
 
-	    if ((lost = rev_branch_of_commit (rl, commits[present])))
+	    if ((lost = rev_branch_of_commit (rl, revisions[present])))
 		fprintf (stderr, " Possible match on %s.", lost->name);
 	    fprintf (stderr, "\n");
 	}
 	if (*tail) {
 	    if (prev)
-		prev->tail = 1;
+		prev->tail = true;
 	} else 
-	    *tail = rev_commit_build (commits, commits[0], nbranch);
+	    *tail = git_commit_build (revisions, revisions[0], nbranch);
     }
     for (n = 0; n < nbranch; n++)
-	if (commits[n])
-	    commits[n]->tailed = false;
-    free (commits);
-    branch->commit = head;
+	if (revisions[n])
+	    revisions[n]->tailed = false;
+    free (revisions);
+    /* PUNNING: see the big comment in cvs.h */ 
+    branch->commit = (cvs_commit *)head;
 }
 
 /*
  * Locate position in tree corresponding to specific tag
  */
 static void
-rev_tag_search(Tag *tag, rev_commit **commits, rev_list *rl)
+rev_tag_search(Tag *tag, cvs_commit **revisions, rev_list *rl)
 {
-	rev_commit_date_sort(commits, tag->count);
-	tag->parent = rev_branch_of_commit(rl, commits[0]);
-	if (tag->parent)
-		tag->commit = rev_commit_locate (tag->parent, commits[0]);
-	if (!tag->commit) {
-		fprintf (stderr, "Unmatched tag %s\n", tag->name);
+    cvs_commit_date_sort(revisions, tag->count);
+    /* tag gets parented with branch of most recent matching commit */
+    tag->parent = rev_branch_of_commit(rl, revisions[0]);
+    if (tag->parent)
+	tag->commit = git_commit_locate (tag->parent, revisions[0]);
+    if (!tag->commit) {
+	announce("tag %s could not be assigned to a commit\n", tag->name);
 #if 0
-		/*
-		 * ESR: Keith's code appeared to be trying to create a
-		 * synthetic commit for unmmatched tags. The comment 
-		 * from "AV" below points at one reason this is probably
-		 * not a good idea.  Better to fail cleanly than risk
-		 * doing somethging wacky to the DAG.
-		 */
-		/* AV: shouldn't we put it on some branch? */
-		tag->commit = rev_commit_build(commits, commits[0], tag->count);
+	/*
+	 * ESR: Keith's code appeared to be trying to create a
+	 * synthetic commit for unmatched tags. The comment 
+	 * from "AV" below points at one reason this is probably
+	 * not a good idea.  Better to fail cleanly than risk
+	 * doing something wacky to the DAG.
+	 */
+	/* AV: shouldn't we put it on some branch? */
+	tag->commit = git_commit_build(revisions, revisions[0], tag->count);
 #endif
-	}
-	tag->commit->tagged = (tag->commit != NULL);
+    }
+    tag->commit->tagged = (tag->commit != NULL);
 }
 
 static void
 rev_ref_set_parent (rev_list *rl, rev_ref *dest, rev_list *source)
 {
-    rev_ref	*sh;
     rev_list	*s;
     rev_ref	*p;
     rev_ref	*max;
@@ -830,6 +837,7 @@ rev_ref_set_parent (rev_list *rl, rev_ref *dest, rev_list *source)
 
     max = NULL;
     for (s = source; s; s = s->next) {
+	rev_ref	*sh;
 	sh = rev_find_head (s, dest->name);
 	if (!sh)
 	    continue;
@@ -849,7 +857,7 @@ rev_ref_set_parent (rev_list *rl, rev_ref *dest, rev_list *source)
 }
 
 
-#if UNUSED
+#ifdef __UNUSED__
 static void
 rev_head_find_parent (rev_list *rl, rev_ref *h, rev_list *lhead)
 {
@@ -865,20 +873,20 @@ rev_head_find_parent (rev_list *rl, rev_ref *h, rev_list *lhead)
 }
 #endif
 
-#if UNUSED
-static int
+#ifdef __UNUSED__
+static bool
 rev_branch_name_is_ancestor (rev_ref *old, rev_ref *young)
 {
     while (young) {
 	if (young->name == old->name)
-	    return 1;
+	    return true;
 	young = young->parent;
     }
-    return 0;
+    return false;
 }
 #endif
 
-#if UNUSED
+#ifdef __UNUSED__
 static rev_ref *
 rev_ref_parent (rev_ref **refs, int nref, rev_list *rl)
 {
@@ -916,7 +924,7 @@ rev_ref_parent (rev_ref **refs, int nref, rev_list *rl)
     for (h = rl->heads; h; h = h->next)
 	if (parent->name == h->name)
 	    return h;
-    fprintf (stderr, "Reference missing in merge: %s\n", parent->name);
+    announce("reference missing in merge: %s\n", parent->name);
     return NULL;
 }
 #endif
@@ -925,17 +933,19 @@ rev_list *
 rev_list_merge (rev_list *head)
 {
     int		count = rev_list_count (head);
-    rev_list	*rl = calloc (1, sizeof (rev_list));
+    int		n; /* used only in progress messages */
+    rev_list	*rl = xcalloc (1, sizeof (rev_list), "list merge");
     rev_list	*l;
     rev_ref	*lh, *h;
     Tag		*t;
-    rev_ref	**refs = calloc (count, sizeof (rev_ref *));
-    int		nref;
+    rev_ref	**refs = xcalloc (count, sizeof (rev_ref *), "list merge");
 
     /*
      * Find all of the heads across all of the incoming trees
      * Yes, this is currently very inefficient
      */
+    progress_begin("Find heads...", count);
+    n = 0;
     for (l = head; l; l = l->next) {
 	for (lh = l->heads; lh; lh = lh->next) {
 	    h = rev_find_head (rl, lh->name);
@@ -944,33 +954,44 @@ rev_list_merge (rev_list *head)
 	    else if (lh->degree > h->degree)
 		h->degree = lh->degree;
 	}
+	if (++n % 100 == 0)
+	    progress_jump(n);
     }
+    progress_jump(n);
+    progress_end("done, total revisions %d", total_revisions);
     /*
      * Sort by degree so that finding branch points always works
      */
+    progress_begin("Sorting...", count);
 //    rl->heads = rev_ref_sel_sort (rl->heads);
     rl->heads = rev_ref_tsort (rl->heads, head);
-    if (!rl->heads)
+    if (!rl->heads) {
+	free(refs);
 	return NULL;
+    }
+    progress_end(NULL);
 //    for (h = rl->heads; h; h = h->next)
 //	fprintf (stderr, "head %s (%d)\n",
 //		 h->name, h->degree);
     /*
      * Find branch parent relationships
      */
+    progress_begin("Find branch parent relationships...", count);
     for (h = rl->heads; h; h = h->next) {
 	rev_ref_set_parent (rl, h, head);
 //	dump_ref_name (stderr, h);
 //	fprintf (stderr, "\n");
     }
+    progress_end(NULL);
     /*
      * Merge common branches
      */
+    progress_begin("Merge common branches...", count);
     for (h = rl->heads; h; h = h->next) {
 	/*
 	 * Locate branch in every tree
 	 */
-	nref = 0;
+	int nref = 0;
 	for (l = head; l; l = l->next) {
 	    lh = rev_find_head (l, h->name);
 	    if (lh)
@@ -978,25 +999,33 @@ rev_list_merge (rev_list *head)
 	}
 	if (nref)
 	    rev_branch_merge (refs, nref, h, rl);
+	progress_step();
     }
+    progress_end(NULL);
     /*
      * Compute 'tail' values
      */
+    progress_begin("Compute tail values...", NO_MAX);
     rev_list_set_tail (rl);
+    progress_end(NULL);
 
     free(refs);
     /*
      * Find tag locations
      */
+    progress_begin("Find tag locations...", NO_MAX);
     for (t = all_tags; t; t = t->next) {
-	rev_commit **commits = tagged(t);
+	cvs_commit **commits = tagged(t);
 	if (commits)
 	    rev_tag_search(t, commits, rl);
 	else
-	    fprintf (stderr, "lost tag %s\n", t->name);
+	    announce("internal error - lost tag %s\n", t->name);
 	free(commits);
     }
+    progress_end(NULL);
+    progress_begin("Validate...", NO_MAX);
     rev_list_validate (rl);
+    progress_end(NULL);
     return rl;
 }
 
@@ -1032,13 +1061,13 @@ rev_file_free_marked (void)
 }
 
 rev_file *
-rev_file_rev (char *name, cvs_number *n, time_t date)
+rev_file_rev (char *name, cvs_number *n, cvstime_t date)
 {
-    rev_file	*f = calloc (1, sizeof (rev_file));
+    rev_file	*f = xcalloc (1, sizeof (rev_file), "allocating file rev");
 
     f->name = name;
     f->number = *n;
-    f->date = date;
+    f->u.date = date;
     return f;
 }
 
@@ -1049,9 +1078,9 @@ rev_file_free (rev_file *f)
 }
 
 static void
-rev_commit_free (rev_commit *commit, int free_files)
+rev_commit_free (cvs_commit *commit, int free_files)
 {
-    rev_commit	*c;
+    cvs_commit	*c;
 
     while ((c = commit)) {
 	commit = c->parent;
@@ -1064,7 +1093,7 @@ rev_commit_free (rev_commit *commit, int free_files)
     }
 }
 
-void
+static void
 rev_head_free (rev_ref *head, int free_files)
 {
     rev_ref	*h;
@@ -1089,11 +1118,12 @@ void
 rev_list_validate (rev_list *rl)
 {
     rev_ref	*h;
-    rev_commit	*c;
+    git_commit	*c;
     for (h = rl->heads; h; h = h->next) {
 	if (h->tail)
 	    continue;
-	for (c = h->commit; c && c->parent; c = c->parent) {
+	/* PUNNING: see the big comment in cvs.h */ 
+	for (c = (git_commit *)h->commit; c && c->parent; c = c->parent) {
 	    if (c->tail)
 		break;
 //	    assert (time_compare (c->date, c->parent->date) >= 0);
@@ -1106,7 +1136,7 @@ rev_list_validate (rev_list *rl)
  */
 
 static rev_file_list *
-rev_uniq_file (rev_commit *uniq, rev_commit *common, int *nuniqp)
+rev_uniq_file (git_commit *uniq, git_commit *common, int *nuniqp)
 {
     int	i, j;
     int nuniq = 0;
@@ -1117,8 +1147,8 @@ rev_uniq_file (rev_commit *uniq, rev_commit *common, int *nuniqp)
     for (i = 0; i < uniq->ndirs; i++) {
 	rev_dir	*dir = uniq->dirs[i];
 	for (j = 0; j < dir->nfiles; j++)
-	    if (!rev_commit_has_file (common, dir->files[j])) {
-		fl = calloc (1, sizeof (rev_file_list));
+	    if (!git_commit_has_file (common, dir->files[j])) {
+		fl = xcalloc (1, sizeof (rev_file_list), "rev_uniq_file");
 		fl->file = dir->files[j];
 		*tail = fl;
 		tail = &fl->next;
@@ -1129,23 +1159,23 @@ rev_uniq_file (rev_commit *uniq, rev_commit *common, int *nuniqp)
     return head;
 }
 
-int
+bool
 rev_file_list_has_filename (rev_file_list *fl, char *name)
 {
     for (; fl; fl = fl->next)
 	if (fl->file->name == name)
-	    return 1;
-    return 0;
+	    return true;
+    return false;
 }
 
 /*
- * Generate a diff between two commits. Either may be NULL
+ * Generate a diff between two gitspace commits. Either may be NULL
  */
 
 rev_diff *
-rev_commit_diff (rev_commit *old, rev_commit *new)
+git_commit_diff (git_commit *old, git_commit *new)
 {
-    rev_diff	*diff = calloc (1, sizeof (rev_diff));
+    rev_diff	*diff = xcalloc (1, sizeof (rev_diff), __func__);
 
     diff->del = rev_uniq_file (old, new, &diff->ndel);
     diff->add = rev_uniq_file (new, old, &diff->nadd);
@@ -1172,3 +1202,4 @@ rev_diff_free (rev_diff *d)
     free (d);
 }
 
+/* end */
